/* Copyright (C) 2002 Jean-Marc Valin 
   File: encode.h

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


   Portions Copyright 2002, John Edwards <john.edwards33@ntlworld.com>
 */

#ifndef __ENCODE_H
#define __ENCODE_H

#include <stdio.h>
#include <windows.h>

typedef void TIMER;
typedef long (*audio_read_func)(void *src,
                                float **buffer,
                                int samples);

typedef void (*progress_func)(long totalsamples,
                              long samples);

typedef void (*enc_end_func)(char *fn,
                             double time,
                             int rate, 
                             long samples,
                             long bytes);

typedef void (*enc_start_func)(char *fn,
                               char *outfn,
                               int bitrate, 
                               float quality,
                               int managed);

typedef void (*error_func)(char *errormessage);

void *timer_start(void);

double timer_time(void *);

void timer_clear(void *);

int create_directories(char *);

void update_statistics_full(char *fn,
                            long total,
                            long done,
                            double time);

void update_statistics_notime(char *fn,
                              long total,
                              long done,
                              double time);

void update_statistics_null(char *fn,
                            long total,
                            long done,
                            double time);

void start_encode_full(char *fn,
                       char *outfn,
                       int bitrate,
                       float quality, 
                       int managed);

void start_encode_null(char *fn,
                       char *outfn,
                       int bitrate,
                       float quality,
                       int managed);

void final_statistics(char *fn,
                      double time,
                      int rate,
                      long total_samples,
                      long bytes);

void final_statistics_null(char *fn,
                           double time,
                           int rate,
                           long total_samples,
                           long bytes);

void encode_error(char *errmsg);


typedef struct
{
	char *outfile;

	int complexity;
	int frames;
	int max_bitrate;
	int abr_bitrate;
	int vbr;
	int vad;
	int dtx;

	int quality_coefficient;

	int resamplefreq;
	int downmix;

	unsigned int serial;
	int logerr;
} oe_options;

typedef struct
{
	unsigned int serialno;

	audio_read_func read_samples;
	progress_func progress_update;
	enc_end_func end_encode;
	enc_start_func start_encode;
	error_func error;
	
	void *readdata;

	long total_samples_per_channel;
	int channels;
	long rate;
	int samplesize;
	int endianness;
	int resamplefreq;

	int complexity;
	int frames;
	int max_bitrate;
	int abr_bitrate;
	int vbr;
	int vad;
	int dtx;
	int quality_coefficient;
	int downmix;

	FILE *out;
	char *filename;
	char *infilename;
} oe_enc_opt;


int oe_encode(oe_enc_opt *opt);

typedef struct
{
	progress_func   progress_update;
	error_func      error;
	int             man_gain;	
	int             decode_mode;
	int	        limiter;
	int	        dither;
	int             del_input;
	int             format;
	double          wrap_prev_pos;
	double          wrap_prev_neg;
	int             conv_factor;
	char            *input_spx;
	char            *out_wave;
} oe_dec_opt;


int oe_decode(oe_dec_opt *opt);

typedef struct {
	char	TitleFormat[32];
	int	window_x;
	int	window_y;
	int	always_on_top;
	int	logerr;
	int	quality;
	int	complexity;
	int	frames;
	int	max_bitrate;
	int	abr_bitrate;
	int	vbr;
	int	vad;
	int	dtx;
	int	delete_input;
	int	volume;
	int	limiter;
	int	dither;
	int     decode_mode;
	int     wav_format;
	int     resample_rate;
	int     resample_mode;
	int     downmix;
	int     out_dir_mode;
	char    out_dir[MAX_PATH];
} SettingsSPEEX;

/************************ GLOBALS *************************/
extern SettingsSPEEX iniSettings;

#endif /* __ENCODE_H */
